//IO
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
//Http
import java.net.URL;
import java.net.URLConnection;



//Some Gui
import javax.swing.JOptionPane;
//GUI-Thread Library
import javax.swing.SwingWorker;
 
/**
 * Execute file download in a background thread and update the progress.
 * */
public class UrlDownload extends SwingWorker<Void, Void> {
 
    URL objectUrl;
    String localFilePath;
    MADGui madGui;
    boolean verbose;
    
    //Http connection String
    static final String staticAsterUrl = "http://e4ftl01.cr.usgs.gov/ASTT/";
    
    
    public UrlDownload(MADGui madGui,URL objectUrl, String saveDir, boolean verbose) {
    	
    	this.madGui = madGui;
        this.objectUrl = objectUrl;
        this.localFilePath = saveDir; 
        this.verbose = verbose;
        
    }    
    
    //This method runs in background. Inherited from SwingWorker.
	@Override
    protected Void doInBackground() throws Exception {
		try{
			URLConnection urlConnection = objectUrl.openConnection();
			InputStream inputStream = urlConnection.getInputStream();
			FileOutputStream outputStream = new FileOutputStream(new File(localFilePath));
		
			byte[] buffer = new byte[4096];
			int bytesRead = -1;
			long totalBytesRead = 0;
			int percentCompleted = 0;
		
			while ((bytesRead = inputStream.read(buffer)) != -1) {
				outputStream.write(buffer, 0, bytesRead);
				totalBytesRead += bytesRead;
				percentCompleted = (int) (totalBytesRead * 100 /urlConnection.getContentLengthLong());
				setProgress(percentCompleted);//The Progress Bar notification
			}

			outputStream.close();
			inputStream.close();
		}catch(Exception err){
			JOptionPane.showMessageDialog(null,
                    "Error downloading file: " + err.getMessage(), "Connection Error",
                    JOptionPane.ERROR_MESSAGE);
            setProgress(0);
            cancel(true);
            madGui.infoLabel.setText("Try another download directory or check your internet connection");
		}
		return null;

    }
	
	//Overrides the "Done" methods, that executes a command on the EDT. In this case shows a dialog.
	@Override
    protected void done() {
		
        if (!isCancelled() && !localFilePath.contains(".xml") && verbose) {
 
        	JOptionPane.showMessageDialog(MADGui.frame,"Download Succesful.\nCheck your download folder (Home by default)", "Download Finished",1);
        }
	}
}